/* filtration and image proccesing program of camera snaps
LiNe 8.3.1999 - 16.3.1999 + 13.6.1999 - 29.6.1999 + 25.9.1999 
+ 9.10.1999 + 30.3.2000 - change to Bt878
+ 13.5.2000 - dynamic control of brightness thershold level in primary filter
14.5.2000 primary picture recording change
21.5.2000 - improved autocalibration - new reference image stability testing*/
/* 30.5.2000 - 5.6.2000 - integrared version with variant translation for all variants
8.9.2001 - new camera no. 3 
19.-22.8.2004 - comment translation*/

/* Copyright (C) 1999-2005  Libor Neumann (LiNe)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.*/


/* direct transformation of position change into parameter space 
added direct camera snaping with Video4Linux (bttv) */
/* preliminary image filtration before gradient computation */

/* fixed camera picture size 576 rows and 768 columns*/
/* value 0 is balck, value 255 - FF is white
origin 0,0 is in upper left corner */
/* based on image processing example from AdOcolus Student edition
the tranformation is totaly different */
/*  Only square image 512x512 */
/* boundary thinning computed by gradient uses adaptive 
thershold level based on maximum gradient */
/* Based on only two parameter transformation i.e. X axis and Y axis,
shift rotation is negligibled 

Transformation computing uses reference image in form of thinned gradient

Measured image analysis uses 256x256 window placed in image centre

Parameter space is only 256x256 - it brings accepted computing time 
and exluding of out of full image boundary compare. */

/* measure is in infinit loop. Timing is controled by source code variables. 
Filtered and full backup images are writen into files in defined time intervals 
- can be used foe backward focus cecking and other backward analysis */
/* reference image is writed in defined time interval and in autocalibration*/

/* basic version switch */

//#define MERAK 1 // measurement set no. 1
//#define MERAK 2 // measurement set no. 2
#define MERAK 3 // measurement set no. 3

#define PROVOZ 1 // direct measurement on Linuxu
//#define PROVOZ 2 // tuning from files on WNT of W2K
//#define PROVOZ 3 // 1 input file processing like dirrect measure
// in case provoz 3 must be autocejch 1
//#define PROVOZ 4 // image snaping without processing - it works with any autocalibration modes

//#define AUTOCEJCH 1 // autocalibration without testing
#define AUTOCEJCH 2 // autocalibration with testing


typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;
#define  INT     unsigned int
#define PI 3.1415926535
#define  BYTE  unsigned char

struct CGStruc {
   int r;
   int c;
};

struct PolStruc {
   float Min;
   float Max;
};

struct ChnStruc {
   int r;
   int c;
   int i;
};

struct SegStruc {
   int r0;
   int c0;
   int r1;
   int c1;
};

struct LinStruc {
   int r;
   int c;
};

struct  LinStrucH {
   int r0;
   int c0;
   int r1;
   int c1;
   BYTE Inc; 
   BYTE Dir;
} ;



struct StrucStrucBin {
   int r;
   int c;
};

struct StrucStrucGrey {
   int r;
   int c;
   int g;
};

struct EvalStruc {
   float Energy;
   float Contrast;
   float Entropy;
   float Homogen;
};

typedef  struct CGStruc         CGTyp;
typedef  struct PolStruc        PolTyp;
typedef  struct LinStruc        LinTyp;
typedef  struct LinStrucH       LinTypH;
typedef  struct ChnStruc        ChnTyp;
typedef  struct SegStruc        SegTyp;
typedef  struct StrucStrucBin   StrTypB;
typedef  struct StrucStrucGrey  StrTypG;
typedef  struct EvalStruc       EvalTyp;


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <errno.h>

#if PROVOZ == 1 || PROVOZ == 4

	#include <unistd.h>
#endif


#include <time.h>

/* =========================================================================

     Copyright (c) 1998 Regents of Koji OKAMURA
     All rights reserved.

   ========================================================================= */

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#if PROVOZ == 1 || PROVOZ == 4
	#include <sys/ioctl.h>
	#include <sys/mman.h>

	#include <linux/videodev.h>

	#define WIDTH 768 
	#define HEIGHT 576
	#define V4L_DEVICE "/dev/video0"

	struct video_capability vcap;
	struct video_channel    vc;
	struct video_mmap       mm;
	struct video_buffer	vbuf;
	struct video_mbuf	mbuf;
	struct video_picture	pict;
#endif

struct tm	cas;
time_t sec;
#define  IMSIZE   512 /*image size */
#define CONTRAST 65000 /* the right camera contrast seting */


void **ArrAlloc (int rows, int cols, INT size);
void ArrFree (void **array, int rows);

/* accumulator analyze */
int AnalyzeAccu (int ImSize, int AccuSize,
				 int Thres, BYTE **Accu,LinTypH *Lines);

/* Cartesian to polar coordinate transformation */
void CarToPol (int MaxGV, int ImSize, int MagThres,
			   int **DeltaX, int **DeltaY,
			   BYTE ** GradMag,BYTE **GradAng);
/*auxiliary function for CarToPol*/
//int DiscAtan16 (int dy, int dx);
int DiscAtan256 (int dy, int dx);

/* accumulator clean */
void CleanAccu (int ImSize, int AccuSize, int WinSize,
				BYTE **InAccu, BYTE ** OutAccu);

/* gradient computation in X and Y direction with 5x5 mask */
void GradOp5 (int MaxGV, int ImSize,
			  BYTE ** InImage, int ** DeltaX, int ** DeltaY);

/* Hough transformation */
void HoughTrans (int ImSize, int AccuSize,  int MaxGV,
                 BYTE **ThinMag, BYTE **NormMag,
				 int **IntAccu, BYTE **Accu);
/* thinning */
void Thinning (int ImSize, int DeltaDir,
			   BYTE **GradMag, BYTE **GradAng, BYTE **ThinMag);

/* line into image generation */
int GenLine (int y0, int x0, int y1, int x1, 
			 BYTE Patt, BYTE **Obr);

/* input image filter */
void Filtr1 (int TLevel, int ImSize,
			  BYTE ** InImage, BYTE ** OutImage, int *Contrast, int *WhCoef);

/************************  MAIN  *****************************/

/* verze ASCII */
int main(int argc, char **argv)
{
	

    #define ROW 512 /* picture row number */
    #define COL 768 /* picture column number */
    #define PARSIZE 256 /* parameter field size - maximal shift range of snaped image*/
	#define LINELEN  120 /* input file line lenth */
    typedef struct BITMAPFILEHEADER { // bmfh 
        WORD    bfType; 
        DWORD   bfSize; 
        WORD    bfReserved1; 
       WORD    bfReserved2; 
        DWORD   bfOffBits; 
	} BITMAPFILEHEADER; 
	int i,j, k, r,c, Inc;
#if PROVOZ == 2
	int jj;
#endif
	char infilenam[120], outfilenam[120] ,inormfilenam[120] ;
	char onormfilenam[120],initfilenam[120],resfilenam[120] ;
	char bckfilenam[120];
	char line[LINELEN],string[LINELEN],strdate[LINELEN];
	char Auto[2]; // calibration sign
	// * - calibration image write
	// + - candidate to calibration image
	// T - candidate test
	// - - candidate image refuse 
	// " " - nothing - standard measure
	char *stat;
	struct BITMAPFILEHEADER hdr, nhdr;       /* bitmap file-header */ 
	DWORD ihdr[300],nihdr[300];				/* bitmap info-header */ 

	BYTE obrbuff[ROW][COL];			/* the image */
	FILE *sfile;
	DWORD dwTotal;

	int start,cejch;		// program working mode

	int oldsmer, newsmer; // direction check for autocalibration
	int Xo, Yo, dXo, dYo;
#if AUTOCEJCH == 2	
	char onormfilenamT[120] ;
	int dXoT, dYoT, cejchlim;
	LinTypH LinesT[1000];
	int NofLinesT ;
#endif


	/* real time and date */
#if PROVOZ == 4  // direct snaping
//	int interval=1; //basic measure time period in units dependent on condition
	int interval=10; //basic measure time period in units dependent on condition
#else
#if MERAK == 1 //measurement set no. 1
//	int interval=2; //basic measure time period in units dependent on condition
	int interval=1; //basic measure time period in units dependent on condition
#else
#if MERAK == 2 //measurement set no. 2
//	int interval=20; //basic measure time period in units dependent on condition
	int interval=15; //basic measure time period in units dependent on condition
#else  //measurement set no. 3
	int interval=40; //basic measure time period in units dependent on condition
//      int interval=30; //basic measure time period in units dependent on condition
//	int interval=15; //basic measure time period in units dependent on condition
#endif
#endif
#endif
	int intervalo=1; //time period of filterd image write
#if PROVOZ == 2
	int intervalc=5; // calibration time period
#else
	int intervalc=12; // calibration time period
//	int intervalc=5; // calibration time period - tuning
#endif
//	int intervalb=12; // time period of full image write
#if MERAK == 1  //measurement set no. 1
	int intervalb=3; // time period of full image write
#else
#if MERAK == 2 //measurement set no. 2
	int intervalb=1; // time period of full image write
#else   //measurement set no. 3
	int intervalb=1; // time period of full image write
#endif
#endif
	int pruchod=99; // basic measure time period in units dependent on condition
	int pruchodo=99; //time period of filterd image write
	int pruchodc=99; // calibration time period
	int pruchodb=99; // time period of full image write
	int autocejch=0; // thereshold level counter
	int maxautocejch=3; // calibration limit
	int maxdelta=96; //maximal difference limit
	int tt,tto,ttc,ttb,quotc,cejchuj;
	div_t zz;

	/* image processing variables */
	int MaxGV, ObrOff;
	BYTE ** InImage ;
#if PROVOZ == 2 || PROVOZ == 3
	BYTE ** OutImage ;
#endif
	int ** DeltaX ; int ** DeltaY ;
	int MagThres;
	BYTE ** GradMag; BYTE **GradAng;
    int DeltaDir;
	BYTE **ThinMag; BYTE **NormMag; BYTE **NormMagTest;
	int AccuRows; int AccuCols;
	int **IntAccu; BYTE **Accu;
	int WinSize;
	BYTE ** OutAccu;
	int Thres, NofLines ;
	LinTypH Lines[1000];
	BYTE ** Obr;
#if PROVOZ == 1 || PROVOZ == 4
/* for camera */
	unsigned char* buf;
	int fd;
	int re;
#endif

/* relative gray level to averadge gray level for image filtering max. 100*/
	int TLevel;
/* image parameters +
	maximal contrast and wite ratio in full filtered image */
	int Contrast, WhCoef;
	
	strcpy(Auto," ");
	start = 0; cejch = 0; oldsmer=-256;
/* analyzed area shift in image */
ObrOff = 128;
/* maximal gradient */
MaxGV = 0xFF;
/* realtive thereshold level amplitude value 
- related to maximum ie 0xFF - for lover ampl =0 */
MagThres = 64;
/* direction range used in direction value */
DeltaDir = 32;
/* accumulator size for area transformation */
AccuRows = PARSIZE;  AccuCols = PARSIZE;
/* searched surrouding size in direction X and Y */
WinSize= 20 ;
/* level for selection from accumulator */
Thres = 64 ;
cejch = 0; cejchuj = 0;

/* relative gray level to averadge gray level for image filtering max. 100*/
TLevel = 10; //Bt878
#if AUTOCEJCH == 2
/* maximal diffrence level for autocalibration comparison */
cejchlim = 5; 
#endif


   InImage = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   DeltaX = (int **) ArrAlloc (IMSIZE, IMSIZE, sizeof(int));
   DeltaY = (int **) ArrAlloc (IMSIZE, IMSIZE, sizeof(int));
   DeltaY = (int **) ArrAlloc (IMSIZE, IMSIZE, sizeof(int));
   GradMag = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   GradAng = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   ThinMag = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   NormMag = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   NormMagTest = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
   IntAccu = (int **) ArrAlloc (PARSIZE, PARSIZE, sizeof(int));
   Accu = (BYTE **) ArrAlloc (PARSIZE, PARSIZE, sizeof(BYTE));
   OutAccu = (BYTE **) ArrAlloc (PARSIZE, PARSIZE, sizeof(BYTE));
   Obr = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
#if PROVOZ == 2 || PROVOZ == 3
   OutImage = (BYTE **) ArrAlloc (IMSIZE, IMSIZE, sizeof(BYTE));
#endif



 /*  free
 ArrFree (WrkIm1, IMSIZE);
   ArrFree (IntIm1, IMSIZE);*/
#if PROVOZ == 1
	//printf("Start Filtrace obrazku No9/%d \n",MERAK);
	printf("Image filtration start No9/%d \n",MERAK);
#else

#if PROVOZ == 2
	//printf("Start Filtrace obrazku No9/ladeni \n");
	printf("Image filtration start No9/tuning \n");
#else
#if PROVOZ == 3
	//printf("Start Filtrace obrazku ze souboru No9 \n");
	printf("Image filtration from file start No9 \n");
#else
#if PROVOZ == 4
	//printf("Start Snimani obrazku No9/%d \n",MERAK);
	printf("Image snaping start No9/%d \n",MERAK);
#else
	//printf("Start Filtrace obrazku No9/??? \n");
	printf("Image filtration start No9/??? \n");
#endif
#endif
#endif
#endif

/* file name determine */
/* command line: command [-s]*/
		/* standard run */
		strcpy (infilenam, "init.bmp");
		strcpy (resfilenam,"result.txt");
		strcpy (initfilenam, "init.txt");
		/* reference image file name is in file */
		
		/* init input file txt open*/
		sfile = fopen( initfilenam,"rb" );
		if (errno == ENOENT) {
			//printf( "Vstupni soubor nenalezen\n" );
			printf( "Input file did not find\n" );
			return 1;}
		if (sfile == NULL) {
			//printf( "Vstupni soubor neotevren\n" );
			printf( "Input file was not opened\n" );
		return 1;}

		stat = fgets (line ,LINELEN, sfile );

		strcpy (inormfilenam,line);

	 	 /* Close stream */
   		if( fclose( sfile ) )
     			 printf( "The file 'data' was not closed\n" );
		 
		/* one parametr */
		if ( argc == 2 )  
			{
			/* if exists switch -s - start + calibration
			in other cases all is in init file */
			if ( strcmp (argv[1] , "-s") == 0 || 
				strcmp (argv[1] , "-S") == 0 )
				{
			/* global init is asked
			only one image is snaped and output log file is generated +
			reference image file is stored - no regular measure */ 
				start = 1; 
				}
				else { 
#if PROVOZ == 3
					/* command line: command filenam [-c] | [-s]*/
					
					strcpy (infilenam, argv[1]);
		}}
				else
					if (argc == 3)
						/*  2 arguments - filenam + switch */
						if ( strcmp (argv[2] , "-c") == 0 || 
							strcmp (argv[2] , "-C") == 0 )
					{
					strcpy (infilenam, argv[1]);
					cejch = 1; // -c new calibration from input line
						}
						else 
					/*  -s - start */
					if ( strcmp (argv[2] , "-s") == 0 || 
						strcmp (argv[2] , "-S") == 0 )
						{
	   				/* global init is asked
					only one image is snaped and output log file is generated +
					reference image file is stored - no regular measure */ 
							strcpy (infilenam, argv[1]);
						start = 1; 
						}
			else {
					//printf( "Chybny prepinac\n");
					printf( "Wrong switch\n");
					return 1;}
#else

					//printf( "Chybny prepinac\n");
					printf( "Wrong switch\n");
					return 1;}
			}
#endif

#if PROVOZ == 1 || PROVOZ == 4
		/* init input file bmp open*/
		sfile = fopen( infilenam,"rb" );
		if (errno == ENOENT) {
			//printf( "Vstupni soubor nenalezen\n" );
			printf( "Input file did not find\n" );
			return 1;}
		if (sfile == NULL) {
			//printf( "Vstupni soubor neotevren\n" );	
			printf( "Input file was not opened\n" );
		return 1;}

	/* start */

	 //printf("Start nacitani obrazku\n");
	 printf("Image read start\n");


	 i = fread( &hdr.bfType,2,1,sfile  );
	 i = fread( &hdr.bfSize,4,1,sfile  );
	 i = fread( &hdr.bfReserved1,2,1,sfile  );
	 i = fread( &hdr.bfReserved2,2,1,sfile  );
	 i = fread( &hdr.bfOffBits,4,1,sfile  );

	/* I know basic values now 


    hdr.bfSize - file size in Byte
 
    hdr.bfOffBits - ofset to 1.byte of the map */

	dwTotal = hdr.bfSize - hdr.bfOffBits ;

		/* size value checking */

		if (hdr.bfSize != 394294) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}

		if (hdr.bfOffBits != 1078) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}


	/* info header read as it is */
	
		k = fread( &ihdr,4,266, sfile );

	/* content read asi it is */
	 k = fread( &obrbuff,768,512, sfile );
		if ( ferror (sfile)) {	
			//printf( "Chyba pri cteni obr info\n" );
			printf( "Error in picture info\n" );
		return 1;}
		if ( feof (sfile)) {	
			//printf( "Neocekavany konec souboru %i\n",i );
			printf( "Unexpected end of file %i\n",i );
		return 1;}
     
	 	 /* Close stream */
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );

 	 //printf("Obrazek nacten \n");
 	 printf("Picture is read \n");
#endif
/* Read picture is used only for the right format bmp parameters determination */

	 
if (start == 0 ) {
	
	/* reference image read in case of continued measure */
	 //printf("Start nacitani referencniho obrazku\n");
	 printf("Reference image read start\n");
	 		sfile = fopen( inormfilenam,"rb" );
		if (errno == ENOENT) {
			//printf( "Vstupni soubor nenalezen\n" );
			printf( "Input file did not find\n" );
			return 1;}
		if (sfile == NULL) {
			//printf( "Vstupni soubor neotevren\n" );
			printf( "Input file was not opened\n" );
		return 1;}

	 i = fread( &nhdr.bfType,2,1,sfile  );
	 i = fread( &nhdr.bfSize,4,1,sfile  );
	 i = fread( &nhdr.bfReserved1,2,1,sfile  );
	 i = fread( &nhdr.bfReserved2,2,1,sfile  );
	 i = fread( &nhdr.bfOffBits,4,1,sfile  );

	dwTotal = nhdr.bfSize - nhdr.bfOffBits ;

		/* size value checking */
		if (nhdr.bfSize != 394294) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}

		if (nhdr.bfOffBits != 1078) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}
//		header
		k = fread( &nihdr,4,266, sfile );
//		content
		k = fread( &obrbuff,768,512, sfile );
		if ( ferror (sfile)) {	
			//printf( "Chyba pri cteni obr info\n" );
			printf( "Error in picture info\n" );
		return 1;}
		if ( feof (sfile)) {	
			//printf( "Neocekavany konec souboru %i\n",i );
			printf( "Unexpected end of file %i\n",i );
		return 1;}
     
//	 	  Close stream 
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );

 	 //printf("Referencni obrazek nacten \n");
 	 printf("Reference image read end \n");
// field filling - the same shift as in analysed image case 
  for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
		NormMag[r][c]= obrbuff[r][c+ObrOff];

//}
/* result read - enable to continue */
 
/* read output log file until the end */
//printf("Nacteni starych vysledku\n");
printf("Old result read\n");
	 sfile = fopen( resfilenam, "rt" );

	 		do
		{
			stat = fgets (line ,LINELEN, sfile );
#if PROVOZ == 3  // another write format
			k=sscanf(line,"%s %d %d %d %d  ",
			 string, &Xo, &Yo, &dXo, &dYo); 
#else
			k=sscanf(line,"%s %s %d %d %d %d  ",
			 string, string, &Xo, &Yo, &dXo, &dYo); 
#endif


		} while (!feof (sfile));

   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );
/* values from the last line will be used for calculations */
   Xo -=dXo; Yo -=dYo; // Xo,Yo absolute reference position
}

#if PROVOZ == 1 || PROVOZ == 4
/* Camera snaping init */

  fd = open(V4L_DEVICE, O_RDWR);
  if(fd<=0){
    perror("open");
    exit;
  }

  if(ioctl(fd, VIDIOCGCAP, &vcap)<0){
    perror("VIDIOCGCAP");
    exit;
  }

  //fprintf(stderr,"Inicializace : %s\n",vcap.name);
  fprintf(stderr,"Initialization : %s\n",vcap.name);

#if MERAK == 1
  /* input seting -Bt848 =0  */
   vc.channel=0; 
#endif
#if MERAK == 2
  /* input seting -Bt878 =1  */
   vc.channel=1; 
#endif
#if MERAK == 3
  /* input seting -Bt878 =1  */
   vc.channel=1; 
#endif

   if(ioctl(fd, VIDIOCGCHAN, &vc)<0){
      perror("VIDIOCGCHAN");
      exit;
    }

if(ioctl(fd, VIDIOCSCHAN, &vc) < 0) {
  perror("VIDIOCCAPTURE");
  exit;
}

/* contrast seting */ 
   if(ioctl(fd, VIDIOCGPICT, &vc)<0){
      perror("VIDIOCGPICT");
      exit;
    }

pict.contrast = CONTRAST;
pict.brightness = CONTRAST;
printf ("Kontrast ++ %i \n", pict.brightness);

if(ioctl(fd, VIDIOCSPICT, &vc) < 0) {
  perror("VIDIOCSPICT");
  exit;
}

/* buffer and image size seting */

if(ioctl(fd, VIDIOCGFBUF, &vbuf) < 0) {
  perror("VIDIOCGFBUF");
  exit;
}
vbuf.height=HEIGHT;
vbuf.width=WIDTH;
vbuf.bytesperline=WIDTH;
if(ioctl(fd, VIDIOCSFBUF, &vbuf) < 0) {
  perror("VIDIOCSFBUF");
  exit;
} 

if(ioctl(fd, VIDIOCGMBUF, &mbuf) < 0) {
  perror("VIDIOCGMBUF");
  exit;
}

buf = (unsigned char*) mmap (0, mbuf.size, PROT_READ|PROT_WRITE, MAP_SHARED, fd,0);
if((int)buf < 0) {
  perror("mmap");
  exit;
}
// waiting for camera
for (i=0; i< 10000000; i++) k=0;
//printf("Inicializace kamery skoncena \n");
printf("Camera initialization end \n");
#endif

/* initialization ended */
/******************************************************************/
#if PROVOZ == 1 || PROVOZ == 4
/* measure cycle */
do {
#endif
#if PROVOZ == 2
	/* fixed number of measure cycle */
	for (jj=0; jj< 5; jj++) {
#endif
/* waiting for the right time */
time(&sec);
cas = *localtime(&sec);
//printf("Cas: %02i:%02i:%02i\r",cas.tm_hour,cas.tm_min,cas.tm_sec);
//printf("Cas: %02i:%02i:%02i\n",cas.tm_hour,cas.tm_min,cas.tm_sec);
printf("Time: %02i:%02i:%02i\n",cas.tm_hour,cas.tm_min,cas.tm_sec);
#if PROVOZ == 4  //direct snaping is in seconds
/* time interval is in seconds */
tt=cas.tm_sec;
#else
#if MERAK == 1 //measurement set no. 1
/* time interval is in minutes */
tt=cas.tm_min;
#else
#if MERAK == 2  //measurement set no. 2
/* merim inerval v sec */
tt=cas.tm_sec;
#else  //measurement set no. 3
/* time interval in second but over minute range */
tt=cas.tm_sec+(cas.tm_min*60);
#endif
#endif
#endif
zz=div(tt,interval);
/* time interval is now or start */
#if PROVOZ == 1 || PROVOZ == 4
if (((zz.rem == 0) && (pruchod != tt)) || (start == 1)) 
{
#endif
pruchod=tt;

printf("Cas %i: %i: %i \n",cas.tm_hour,cas.tm_min,cas.tm_sec);
#if PROVOZ == 1 || PROVOZ == 4
/* image preparation*/
 mm.frame  = 0;
 mm.height = HEIGHT;
 mm.width  = WIDTH;
 mm.format = VIDEO_PALETTE_GREY;

 

 if(ioctl(fd, VIDIOCMCAPTURE, &mm)<0){
    perror("VIDIOCMCAPTURE");
    exit;
  }

  if(ioctl(fd, VIDIOCSYNC, &mm.frame)<0){
    perror("VIDIOCSYNC");
    exit;
  }
#endif
#if PROVOZ == 2 || PROVOZ == 3
  		/* input bmp file open - instead of camera input*/
		sfile = fopen( infilenam,"rb" );
		if (errno == ENOENT) {
			//printf( "Vstupni soubor nenalezen\n" );
			printf( "Input file did not find\n" );
			return 1;}
		if (sfile == NULL) {
			//printf( "Vstupni soubor neotevren\n" );
			printf( "Input file was not opened\n" );
		return 1;}

	/* start */

	 //printf("Start nacitani vstupniho obrazku\n");
	 printf("Input image read start");


	 i = fread( &hdr.bfType,2,1,sfile  );
	 i = fread( &hdr.bfSize,4,1,sfile  );
	 i = fread( &hdr.bfReserved1,2,1,sfile  );
	 i = fread( &hdr.bfReserved2,2,1,sfile  );
	 i = fread( &hdr.bfOffBits,4,1,sfile  );

	/* I know basic values now 


    hdr.bfSize - file size in Byte
 
    hdr.bfOffBits - ofset to 1.byte of the map */

	dwTotal = hdr.bfSize - hdr.bfOffBits ;

		/* size value checking */
		if (hdr.bfSize != 394294) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}

		if (hdr.bfOffBits != 1078) {
			//printf( "Chybny format obrazku\n" );
			printf( "Wrong picture format\n" );
		return 1;}


	/* info header read as it is */
	
		k = fread( &ihdr,4,266, sfile );

	/* content read asi it is */
	 k = fread( &obrbuff,768,512, sfile );
		if ( ferror (sfile)) {	
			//printf( "Chyba pri cteni obr info\n" );
			printf( "Error in picture info\n" );
		return 1;}
		if ( feof (sfile)) {	
			//printf( "Neocekavany konec souboru %i\n",i );
			printf( "Unexpected end of file %i\n",i );
		return 1;}
     
	 	 /* Close stream */
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );

 	 //printf("Obrazek nacten \n");
 	 printf("Image read end \n");

#endif

  
#if PROVOZ != 3  
  /* snap time in characters and file name for image snap write */
sprintf(strdate,"%02i-%02i-%4i %02i:%02i:%02i",
	cas.tm_mday,cas.tm_mon+1,cas.tm_year+1900,
	cas.tm_hour,cas.tm_min,cas.tm_sec);
sprintf(outfilenam,"%02i%02i%02i%02i%02i.bmp", 
	cas.tm_mon+1,cas.tm_mday,cas.tm_hour,cas.tm_min,cas.tm_sec);
#endif

#if PROVOZ == 1 
strcpy(onormfilenam,"n");
strcat(onormfilenam,outfilenam);
strcpy(bckfilenam,"b");
strcat(bckfilenam,outfilenam);
#endif
#if PROVOZ == 2
strcpy(onormfilenam,"norm.bmp");
strcpy(bckfilenam,"bck.bmp");
strcpy(outfilenam,"out.bmp");
#endif
#if PROVOZ == 3  // names are derived from input file name  
strcpy(onormfilenam,"n");
strcat(onormfilenam,infilenam);// no image write is needed 
strcpy(strdate,infilenam); //input file name instead of time
#endif
#if PROVOZ == 4 
strcpy(bckfilenam,outfilenam);
#endif
	 
/* computation body */

//printf("Cas snimku: %s \n",strdate);
printf("Snap time: %s \n",strdate);



#if PROVOZ == 1 
/* image processing */ 
for (c=0; c< IMSIZE; c++)
	for (r=0; r< IMSIZE; r++)
		InImage[IMSIZE-r-1][c]=*(buf+(r*WIDTH+c+ObrOff)); // from camera
#endif
#if PROVOZ == 2 || PROVOZ == 3
// field filling - the same shift as in analysed image 
// only tuning 
  for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
		InImage[r][c]= obrbuff[r][c+ObrOff];
#endif

#if PROVOZ != 4
/* first image filtration */
	 //printf("Start filtrace \n");
	 printf("Image filtration start \n");
#if PROVOZ == 1
	Filtr1 (TLevel, IMSIZE, InImage, ThinMag, &Contrast, &WhCoef);
#else
	Filtr1 (TLevel, IMSIZE, InImage, OutImage, &Contrast, &WhCoef);
#endif
// Filtr1 (TLevel, IMSIZE, InImage, ThinMag);
// 	 printf("Konec filtrace \n");
 	 //printf("Konec filtrace Contr=%3d WhCoef=%3d\n",Contrast,WhCoef);
 	 printf("Filtration end Contr=%3d WhCoef=%3d\n",Contrast,WhCoef);
/* gradient computation */
 	 //printf("Start vypoctu gradientu \n");
 	 printf("Gradient computation start \n");
//GradOp5 (MaxGV, IMSIZE, InImage, DeltaX, DeltaY);
#if PROVOZ == 1
	GradOp5 (MaxGV, IMSIZE, ThinMag, DeltaX, DeltaY);
#else
	GradOp5 (MaxGV, IMSIZE, OutImage, DeltaX, DeltaY);
#endif
 	 //printf("Konec vypoctu gradientu \n");
 	 printf("Gradient computation end \n");

/* Cartesian to polar coordinate system conversion */
 	 //printf("Start prevodu do polarnich souradnic\n");
 	 printf("Polar coordinates conversion start\n");
CarToPol (MaxGV,IMSIZE,MagThres,
			   DeltaX,DeltaY,GradMag,GradAng);
 	 //printf("Konec prevodu do polarnich souradnic\n");
 	 printf("Polar coordinates conversion end\n");
/* thinning */
 	 //printf("Start zeslabeni\n");
 	 printf("Thinning start\n");
Thinning (IMSIZE,DeltaDir,
			   GradMag,GradAng,ThinMag);
 	 //printf("Konec zeslabeni\n");
 	 printf("Thinning end\n");
if (start == 0) {
/* Hough transformation */
	 //printf("Start Hough transformace\n");
	 printf("Hough transformation start\n");
HoughTrans (IMSIZE,PARSIZE,MaxGV,
                 ThinMag,NormMag,IntAccu,Accu);
 	 //printf("Konec Hough transformace\n");
 	 printf("Hough transformation end\n");
/* accumulator cleaninig  - only the greatest values 
are selected from square surrounding WinSize x WinSize in parameter space */
	 //printf("Start uklidu akumulatoru\n");
	 printf("Accumulator cleaning start\n");
CleanAccu (IMSIZE,PARSIZE,WinSize,
				Accu,OutAccu);
 	 //printf("Konec uklidu akumulatoru\n");
 	 printf("Accumulator cleaninig end\n");
/*Analyze */
// buffer cleaning
	 for (i=0; i<1000; i++)
	 { Lines[i].r0 = 0; Lines[i].c0 = 0; Lines[i].Inc = 0;};

 	 //printf("Start analyzy\n");
 	 printf("Analyze start\n");
 NofLines = AnalyzeAccu (IMSIZE,PARSIZE,
				 Thres,OutAccu,Lines);
 	 //printf("Konec analyzy\n");
 	 printf("Analyze end\n");
/* Results processing */
 	 //printf("Start zpracovani vysledku\n");
 	 printf("Result processing start\n");
/* processing end */
	strcpy(Auto," ");
 /* processing end in standard mode*/

}

if (start == 0)
/* result write into log file */
{

 /* value Inc sort */
	 for (i=1; i< NofLines; i++)
		 for (j=i; j>0; j--)
		 if (Lines[j].Inc > Lines[j-1].Inc)
		/* change */
	 { r = Lines[j].r0; c = Lines[j].c0; Inc = Lines[j].Inc;
	   Lines[j].r0 = Lines[j-1].r0; Lines[j].c0 = Lines[j-1].c0;
	   Lines[j].Inc = Lines[j-1].Inc;
	   Lines[j-1].r0 = r; Lines[j-1].c0 = c; Lines[j-1].Inc = Inc; 
	 }


 /* result write to the file end */
/*  new values */

//	Xo= Xo-dXo+Lines[0].c0; //Xo, Yo reference absolute value
//	Yo= Yo-dYo+Lines[0].r0;
	dXo= Lines[0].c0;
	dYo= Lines[0].r0;


/* autocalibration condition test
- continuos thereshold level overstep minimaly in one axis 
and in one direction and direction must be maximally in two ajacent fields 
in circle dividet into 16 filelds */
	newsmer=DiscAtan256(dYo,dXo);

#if PROVOZ != 3  // autocalibration is nonsense
#if PROVOZ == 2
/* time interval in minutes */
ttc=cas.tm_min;
//ttc=cas.tm_hour;
zz=div(ttc,intervalc);
quotc=zz.quot; //my own variable to prevent overwrite
		/* autocalibration condition */
if (((zz.rem == 0) && (pruchodc != zz.quot)))  // only time condition 
//	if (start == 0)
#else
#if AUTOCEJCH == 2
/* two conditions for autocalibration:
1- Time limit - periodical autcalibration - starts autocalibration process until autocalibration is succesfull
2- Space limit - great position change -  starts autocalibration process until autocalibration is succesfull or
position returns */

/* time interval in hours */
//ttc=cas.tm_min; //tuning
ttc=cas.tm_hour;
zz=div(ttc,intervalc);
quotc=zz.quot;
		/* is time or great position change */
if ((zz.rem == 0) && (pruchodc != zz.quot))  cejchuj=1;
if ((cejchuj == 1)  ||
	(( abs(dXo) > maxdelta || abs(dYo) > maxdelta) &&
		( oldsmer <= -256 || abs(oldsmer - newsmer) < 20 ||
		abs(oldsmer - newsmer) > 230 )))
			/* it is new or in the same direction*/
#else
		/* is only position change */
	if (
	(( abs(dXo) > maxdelta || abs(dYo) > maxdelta) &&
		( oldsmer <= -256 || abs(oldsmer - newsmer) < 20 ||
		abs(oldsmer - newsmer) > 230 )))
			/* it is new or in the same direction*/
#endif
#endif
			{
			autocejch++;
			strcpy(Auto,"T");

#if AUTOCEJCH == 1
			if (autocejch == 1) {
				oldsmer = newsmer ;
				strcpy(Auto,"+");
			}
			}
		else
		/* lower position change or great direction change */
		if (autocejch >0 ) {
		autocejch = 0;
		oldsmer = -256;
		strcpy(Auto,"-");
		}

#else

			if (autocejch == 1) {
				oldsmer = newsmer ;
				// field fill from memory to memory - new reference 
				for (r=0; r< IMSIZE; r++) 
					for (c=0; c< IMSIZE; c++) 
						NormMagTest[r][c]= ThinMag[r][c];
				//new reference position write
				dXoT=dXo; dYoT=dYo;
				strcpy(Auto,"+");
				strcpy(onormfilenamT,onormfilenam); //reference candidate file name write
			}


			else {
				/* distance test useing reference candidate */
					//printf("Start Hough transformace nove reference\n");
					printf("New reference Hough transformation start\n");
					HoughTrans (IMSIZE,PARSIZE,MaxGV,
							ThinMag,NormMagTest,IntAccu,Accu);
 					//printf("Konec Hough transformace nove reference\n");
					printf("New reference Hough transformation end\n");
					/* accumulator cleaninig  - only the greatest values 
					are selected from square surrounding WinSize x WinSize in parameter space */
					//printf("Start uklidu akumulatoru nove reference\n");
					printf("New reference accumulator cleaning start\n");
					CleanAccu (IMSIZE,PARSIZE,WinSize,
						Accu,OutAccu);
 					//printf("Konec uklidu akumulatoru nove reference\n");
					printf("New reference accumulator cleaning end\n");
				/*Analyze */
				// test buffer cleaning
					for (i=0; i<1000; i++)
						{ LinesT[i].r0 = 0; LinesT[i].c0 = 0; LinesT[i].Inc = 0;};

 					//printf("Start analyzy nove reference\n");
 					printf("New reference analyze start\n");
					NofLinesT = AnalyzeAccu (IMSIZE,PARSIZE,
						Thres,OutAccu,LinesT);
					//printf("Konec analyzy nove reference\n");
  					printf("New reference analyze end\n");
					/* result processing */
 					//printf("Start zpracovani vysledku nove reference\n");
 					printf("New reference result processing start\n");
					/* end of processing */

					/* value Inc sort */
					for (i=1; i< NofLinesT; i++)
						for (j=i; j>0; j--)
							if (LinesT[j].Inc > LinesT[j-1].Inc)
							/* change */
						{ r = LinesT[j].r0; c = LinesT[j].c0; Inc = LinesT[j].Inc;
						LinesT[j].r0 = LinesT[j-1].r0; LinesT[j].c0 = LinesT[j-1].c0;
						LinesT[j].Inc = LinesT[j-1].Inc;
						LinesT[j-1].r0 = r; LinesT[j-1].c0 = c; LinesT[j-1].Inc = Inc; 
					}

 					//printf("Start porovnani vysledku nove reference\n");
 					printf("New reference result comparison start\n");
										
					//result comparison
					if ((abs(dXoT + LinesT[0].c0 - dXo) > cejchlim) ||
						(abs(dYoT + LinesT[0].r0 - dYo) > cejchlim))
						//wrong calibration - must be another autocalibration 
								{
								autocejch = 0;
								oldsmer = -256;
								strcpy(Auto,"-");
							};


			}



			}
		else
		/* lower position change or great direction change */
		if (autocejch >0 ) {
		autocejch = 0;
		oldsmer = -256;
		strcpy(Auto,"-");
		}
#endif
#endif		
/* test if there are condition for another reference */

#if PROVOZ == 3
	if (cejch == 1)
	{
	strcpy(Auto,"*");
	Xo +=dXo; Yo +=dYo;
    dXo = 0; dYo = 0; 
#else
#if AUTOCEJCH == 2
if (autocejch >=maxautocejch)
	{
	strcpy(onormfilenam,onormfilenamT); //return to the candidate file name

#else
/* time interval in hours */
//ttc=cas.tm_min; //tuning
ttc=cas.tm_hour;
zz=div(ttc,intervalc);
quotc=zz.quot;
		/* autocalibration condition */
if (((zz.rem == 0) && (pruchodc != zz.quot))  || (autocejch >=maxautocejch))
	{
#endif

	pruchodc=quotc;	// prevent repetitive calibration if previous calibration exists		


/* reference image write in case of initialization or autocalibration */
	cejch=1; autocejch = 0;
	strcpy(Auto,"*");
 /* new reference value used for next reference position change analysis */
#if AUTOCEJCH == 2
   	Xo +=dXoT; Yo +=dYoT;
	dXo -=dXoT; dYo -=dYoT; 
#else
	Xo +=dXo; Yo +=dYo;
    dXo = 0; dYo = 0; 
#endif
#endif
	}

/* read output log file until the end */
	 sfile = fopen( resfilenam, "a+t" );
	 /* result write */
//    fprintf(sfile, "%12s %5d %5d %4d %4d %4d  %4d %4d %4d  %4d %4d %4d %3d %3d\n",
//	 strdate, Xo, Yo, dXo, dYo, NofLines,
    fprintf(sfile, "%19s %5d %5d %4d %4d %4d  %4d %4d %4d  %4d %4d %4d %3d %3d %1s\n",
	 strdate, Xo+dXo, Yo+dYo, dXo, dYo, NofLines,
	 Lines[1].c0, Lines[1].r0, Lines[1].Inc,
	 Lines[2].c0, Lines[2].r0, Lines[2].Inc, WhCoef,Contrast,Auto);
//	 Lines[2].c0, Lines[2].r0, Lines[2].Inc);

   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );
}
else
/* result log header write */
{
	 //printf("Zacina zakladat log soubor\n");
	 printf("Output log creation start\n");
	 sfile = fopen( resfilenam, "wt" );

#if PROVOZ == 1
	 fprintf(sfile, "Log ObrFiltr No9/%d \n",MERAK);
#else
#if PROVOZ == 2
	 //fprintf(sfile, "Log ObrFiltr No9/ladeni \n");
	 fprintf(sfile, "Log ObrFiltr No9/tuning \n");
#else
#if PROVOZ == 3
	 fprintf(sfile, "Log ObrFiltr No9/file \n");
#else
	 fprintf(sfile, "Log ObrFiltr No9/??? \n");
#endif
#endif
#endif


	 fprintf(sfile, "       Time             X     Y   dX   dY    n    X2   Y2 Inc2    X3   Y3 Inc3  WC Cnt A\n");
//	 fprintf(sfile, "       Time             X     Y   dX   dY    n    X2   Y2 Inc2    X3   Y3 Inc3  WC Cnt\n");
//	 fprintf(sfile, "       Time             X     Y   dX   dY    n    X2   Y2 Inc2    X3   Y3 Inc3 \n");

	  /* result write to the end of file */
 fprintf(sfile, "%19s %5d %5d %4d %4d %4d \n",
// fprintf(sfile, "%12s %5d %5d %4d %4d %4d \n",
	 strdate,0,0,0,0,0);
	cejch=1;

}

#if PROVOZ == 2
//if ( start == 0 )  // tuning
if ( cejch == 1 )
#else
if ( cejch == 1 )
#endif  
/* reference image write */
 
{ 
	 //printf("Zacina zaznam referencniho obrazku \n");
	 printf("Reference image write start \n");
	cejch = 0; cejchuj = 0; oldsmer = -256;
#if PROVOZ == 1 || PROVOZ == 3
	/* reference image file name write into init txt file */
	sfile = fopen( initfilenam, "wt" );
	fprintf(sfile,"%s",onormfilenam);
	fclose(sfile);
#endif
	/* flag for reading from file into memory */
//	cejch = 1;

#if AUTOCEJCH == 1
// reference field filling from memory to memory  
  for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
		NormMag[r][c]= ThinMag[r][c];
#else
// reference field filling from memory to memory  - tested reference candidate 
if (start == 1) // reference field filling from memory to memory  
  for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
		NormMag[r][c]= ThinMag[r][c];
else   //older candidate
  for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
		NormMag[r][c]= NormMagTest[r][c];
#endif
	 /* output filling  -  filled by black  */
for (r=0; r< ROW; r++) 
	for (c=0; c< COL; c++) 
		obrbuff[r][c]= 0x00;
/* Copiing from camera buffer - whole image */
/*for (c=0; c< COL; c++)
  for (r=0; r< ROW; r++)
      obrbuff[ROW-r-1][c]=*(buf+(r*WIDTH+c));
*/
for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
{
//      Gradientmuze byt i zaporny, aby byl videt je nutne zobrazit abs. hodnotu
//		obrbuff[r][c]= (BYTE) abs(DeltaX[r][c]);
//		obrbuff[r][c]= (BYTE) abs(DeltaY[r][c]);
//		Prevod do polarnich souradnic prirazuje malym amplitudam uhel 0 
//		obrbuff[r][c]= (BYTE) GradMag[r][c];
//		obrbuff[r][c+ObrOff]=InImage[r][c]; // ladeni
//		obrbuff[r][c]=0xff;
//		if (GradMag[r][c]> MagThres) obrbuff[r][c]=0xFF;
//			else obrbuff[r][c]=0x00;
//		Pozadi dam umele na stredni hodnotu uhlu
//	    if (GradMag[r][c]> 0) obrbuff[r][c]=(BYTE) GradAng[r][c];
//			else obrbuff[r][c]=0x80;
//		obrbuff[r][c+ObrOff]= (BYTE) ThinMag[r][c];
//	 	obrbuff[r][c+ObrOff]= (BYTE) NormMag[r][c];
//		Zvyrazneni tak, jak ho uvidi transformace 
//		if (ThinMag[r][c+ObrOff]> MagThres) obrbuff[r][c]=0x00;
//		Reference image store

		if (NormMag[r][c]> MagThres) obrbuff[r][c+ObrOff]=0xFF;
			else obrbuff[r][c+ObrOff]=0x00;

//		obrbuff[r][c]= (BYTE) ThinAng[r][c];

}
/*for (r=0; r< AccuCols; r++) 
	for (c=0; c< AccuRows; c++) 
{
//		obrbuff[r][c]= (BYTE) Accu[r][c];
		obrbuff[r][c]= (BYTE) OutAccu[c][r];

}


	 /* processed image write into file */

	 /* output file open */
     /*	sfile = _wfopen( outfilenam, L"w" );*/
	 sfile = fopen( onormfilenam, "wb" );

	 /* file header write - as it was */
	 i = fwrite( &hdr.bfType,2,1,sfile  );
	 i = fwrite( &hdr.bfSize,4,1,sfile  );
	 i = fwrite( &hdr.bfReserved1,2,1,sfile  );
	 i = fwrite( &hdr.bfReserved2,2,1,sfile  );
	 i = fwrite( &hdr.bfOffBits,4,1,sfile  );


	/* file info write - as it is */
		k = fwrite( &ihdr,4,266, sfile );

	/* image content write as it is */
	 k = fwrite( &obrbuff,768,512, sfile );


	 /* ... */
	 /* Close stream */
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );
	 //printf("Konec zaznamu referencniho obrazku \n");
	 printf("Reference image write end \n");
};

#if PROVOZ != 3
/* time interval in hours */
//tto=cas.tm_sec;
tto=cas.tm_hour;
zz=div(tto,intervalo);
/* it is the time */
if ((zz.rem == 0) && (pruchodo != zz.quot) && (start == 0))  
	{
	pruchodo=zz.quot;


/* thinned image gradient write */
 
//{ 
	 //printf("Zacina zaznam sejmuteho obrazku \n");
	 printf("Snaped image write start \n");

	 /* output filling  -  filled by black  */
for (r=0; r< ROW; r++) 
	for (c=0; c< COL; c++) 
		obrbuff[r][c]= 0x00;
/* Copiing from camera buffer - whole image */
/*for (c=0; c< COL; c++)
  for (r=0; r< ROW; r++)
      obrbuff[ROW-r-1][c]=*(buf+(r*WIDTH+c));
*/
for (r=0; r< IMSIZE; r++) 
	for (c=0; c< IMSIZE; c++) 
{
//      Gradientmuze byt i zaporny, aby byl videt je nutne zobrazit abs. hodnotu
//		obrbuff[r][c]= (BYTE) abs(DeltaX[r][c]);
//		obrbuff[r][c]= (BYTE) abs(DeltaY[r][c]);
//		Prevod do polarnich souradnic prirazuje malym amplitudam uhel 0 
//		obrbuff[r][c]= (BYTE) GradMag[r][c];
//		obrbuff[r][c+ObrOff]=InImage[r][c]; // ladeni
#if PROVOZ == 2
		obrbuff[r][c+ObrOff]=OutImage[r][c]; // tuning
#endif
//		obrbuff[r][c]=0xff;
//		if (GradMag[r][c]> MagThres) obrbuff[r][c]=0xFF;
//			else obrbuff[r][c]=0x00;
//		Pozadi dam umele na stredni hodnotu uhlu
//	    if (GradMag[r][c]> 0) obrbuff[r][c]=(BYTE) GradAng[r][c];
//			else obrbuff[r][c]=0x80;
//		obrbuff[r][c+ObrOff]= (BYTE) ThinMag[r][c];
//	 	obrbuff[r][c+ObrOff]= (BYTE) NormMag[r][c];
//		Zvyrazneni tak, jak ho uvidi transformace 
//		if (ThinMag[r][c+ObrOff]> MagThres) obrbuff[r][c]=0x00;
//		Thinned image write
#if PROVOZ == 1
		if (ThinMag[r][c]> MagThres) obrbuff[r][c+ObrOff]=0xFF;
			else obrbuff[r][c+ObrOff]=0x00;
#endif
//		obrbuff[r][c]= (BYTE) ThinAng[r][c];

}
/*for (r=0; r< AccuCols; r++) 
	for (c=0; c< AccuRows; c++) 
{
//		obrbuff[r][c]= (BYTE) Accu[r][c];
		obrbuff[r][c]= (BYTE) OutAccu[c][r];

}


	 /* processed image write into file */

	 /* output file open */
     /*	sfile = _wfopen( outfilenam, L"w" );*/
	 sfile = fopen( outfilenam, "wb" );

	 /* file header write - as it was */
	 i = fwrite( &hdr.bfType,2,1,sfile  );
	 i = fwrite( &hdr.bfSize,4,1,sfile  );
	 i = fwrite( &hdr.bfReserved1,2,1,sfile  );
	 i = fwrite( &hdr.bfReserved2,2,1,sfile  );
	 i = fwrite( &hdr.bfOffBits,4,1,sfile  );


	/* file info write - as it is */
		k = fwrite( &ihdr,4,266, sfile );

	/* image content write as it is */
	 k = fwrite( &obrbuff,768,512, sfile );


	 /* ... */
	 /* Close stream */
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );
	 //printf("Konec zaznamu sejmuteho obrazku \n");
	 printf("Snaped image write end \n");
};


/* time interval in hours */
//ttb=cas.tm_min;
ttb=cas.tm_hour;
zz=div(ttb,intervalb);
/* it is the time */
//if ((zz.rem == 0) && (pruchodb != zz.quot))  
#if PROVOZ == 2
if (start == 0) 
#else
if ((start == 1) || ((zz.rem == 0) && (pruchodb != zz.quot)))
#endif
  
	{
	pruchodb=zz.quot;

//if ((start == 1) || (cas.tm_sec==0) ) 
/* full backup image write - in inicialization or in the defined time */
#endif
#endif  // all processig omiting end for mode (provoz) 4
#if PROVOZ != 3
 
//{ 
	 //printf("Zacina zaznam plneho obrazku \n");
	 printf("Full image write start \n");

	 /* output filling  -  filled by black  */


#if PROVOZ == 1 || PROVOZ == 4
/* Copiing from camera buffer - whole image */
for (c=0; c< COL; c++)
  for (r=0; r< ROW; r++)
      obrbuff[ROW-r-1][c]=*(buf+(r*WIDTH+c));
#endif

#if PROVOZ == 2
	 /* output filling  -  filled by black  */
for (r=0; r< ROW; r++) 
	for (c=0; c< COL; c++) 
		obrbuff[r][c]= 0x00;
/* Copiing from camera buffer - whole image */
for (r=0; r< IMSIZE; r++)
     for (c=0; c< IMSIZE; c++)
 //     obrbuff[ROW-r-1][c]=*(buf+(r*WIDTH+c));
 //     obrbuff[r][c+ObrOff]=ThinMag[r][c];
      obrbuff[r][c+ObrOff]=InImage[r][c];
#endif


	 /* processed image write into file */

	 /* output file open */
	 sfile = fopen( bckfilenam, "wb" );

	 /* file header write - as it was */
	 i = fwrite( &hdr.bfType,2,1,sfile  );
	 i = fwrite( &hdr.bfSize,4,1,sfile  );
	 i = fwrite( &hdr.bfReserved1,2,1,sfile  );
	 i = fwrite( &hdr.bfReserved2,2,1,sfile  );
	 i = fwrite( &hdr.bfOffBits,4,1,sfile  );


	/* file info write - as it is */
		k = fwrite( &ihdr,4,266, sfile );

	/* image content write as it is */
	 k = fwrite( &obrbuff,768,512, sfile );


	 /* ... */
	 /* Close stream */
   if( fclose( sfile ) )
      printf( "The file 'data' was not closed\n" );
	 //printf("Konec zaznamu plneho obrazku \n");
	 printf("Full image write end \n");
#if PROVOZ != 4
};
#endif
#endif

#if PROVOZ == 1 || PROVOZ == 4
/* Linux buffers empty */
sync();

} /* timing loop end */
#if PROVOZ == 1 && MERAK == 1
else

/* more effective waiting per second to the minute end - Linux*/
if (cas.tm_sec < 59) sleep (1);
#endif

/* measure loop end */
} while ( start == 0 ) ;

/* camera */
   	munmap (buf, mbuf.size);
	close(fd);
#endif
#if PROVOZ == 2
	}
#endif
  
	//printf("Konec behu \n");
	printf("End of run \n");
return 0;
 
} 

/*#######################  Array Allocation  ############################*/
void **ArrAlloc (int rows, int cols, INT size)
{
  void **array;
  int i,j;

  array = (void **)malloc (rows * sizeof (int));

  for (i=0; i<rows; i++) {
    array[i] = malloc (cols * size);
    if (array[i] == NULL) {
      printf ("\n*****  ArrAlloc:  OFF HEAP  *****\n");
      for (j=i-1; j>=0; i--) free(array[i]);
      free (array);
      return (NULL);
    }
  }
  return (array);
}

void ArrFree (void **array, int rows)
{
  int i;
  for (i=0; i<rows; i++) free (array[i]);
  free (array);
  return;
}



int AnalyzeAccu (int ImSize, int AccuSize,
				 int Thres, BYTE **Accu,LinTypH *Lines)
{
/* Analyse is adapted to the transformation type */
   int    r,c, NofLines;
   int X,Y;

   NofLines = 0;

   for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         if ((int)Accu[r][c] > Thres) {
   //        Bacward transformation must be linked with transformation 
            Y = (int) r - (AccuSize/2);
            X = (int) c - (AccuSize/2);
            Lines[NofLines].r0 = -Y;
            Lines[NofLines].c0 = -X;
            Lines[NofLines].r1 = 0;
            Lines[NofLines].c1 = 0;
            Lines[NofLines].Inc = Accu[r][c];
            Lines[NofLines].Dir = 0;
            NofLines++;
   }  }  }
   return (NofLines);
}

void CarToPol (int MaxGV, int ImSize, int MagThres,
			   int **DeltaX, int **DeltaY,
			   BYTE ** GradMag,BYTE **GradAng)
{
   /* for small amplitudes is ange set to 0 
	Small amplitude level is realtive to image maximum */
	int  r,c, dX,dY, Mag, MaxMag, MinMag;

   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         GradMag [r][c] = 0;
         GradAng [r][c] = 0;
   }  }

   MaxMag = 0;
   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         dX = DeltaX [r][c];
         dY = DeltaY [r][c];
         Mag = abs(dX) + abs(dY);
         if  (Mag > MaxMag)  MaxMag = Mag;
   }  }
/* MaxMag is used for thereshold level*/
   MinMag = MagThres * MaxMag/MaxGV;
   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         dX = DeltaX [r][c];
         dY = DeltaY [r][c];
         Mag = abs(dX) + abs(dY);

//         if (Mag > MagThres) {
         if (Mag > MinMag) {
            GradMag [r][c] = (BYTE) (((long)Mag * MaxGV) / MaxMag);
			/* my own modification of direction computation algoritm - gives values 0-255 
			adding 64 and masking to BYTE converts angle orientation 
			to be compatible with use in following processing */	
			GradAng [r][c] = (BYTE) ((DiscAtan256 (dY,dX) + 64) & 255); 
}  }  }  }

void CleanAccu (int ImSize, int AccuSize, int WinSize,
				BYTE **InAccu, BYTE ** OutAccu)
{
   BYTE Inc, Max;
   int  r,c, yw,xw, h;
   int  xw0, xw1, yw0,yw1;

   for (r=0; r<AccuSize; r++)
      for (c=0; c<AccuSize; c++)  OutAccu [r][c] = 0;

   h = WinSize>>1;

   for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         Inc = InAccu[r][c];
         if (Inc) {
            Max = 0;
			/* boundary test in direction x and y */
            xw0=c-h; if (xw0<0) xw0=0;
			xw1=c+h; if (xw1>AccuSize) xw1=AccuSize;
            yw0=r-h; if (yw0<0) yw0=0;
			yw1=r+h; if (yw1>AccuSize) yw1=AccuSize;
            for (yw=yw0; yw<yw1; yw++) {
               for (xw=xw0; xw<xw1; xw++) {
                  if (InAccu[yw][xw] > Max)  Max = InAccu[yw][xw];
            }  }
            if (Inc==Max)  OutAccu[r][c] = Inc;
}  }  }  }


void GradOp5 (int MaxGV, int ImSize,
			  BYTE ** InImage, int ** DeltaX, int ** DeltaY)
{
   /* gradient can have positive or negative value */
	long  dXl, dYl;
   int   r,c, dX,dY, gv, y,x, MaxMag;

//   static int Xmask [5][5] = { { -10, -10,  0,  10,  10},
//                               { -17, -17,  0,  17,  17},
//                               { -20, -20,  0,  20,  20},
//                               { -17, -17,  0,  17,  17},
//                               { -10, -10,  0,  10,  10} };
//   static int Ymask [5][5] = { {  10,  17,  20,  17,  10},
//                               {  10,  17,  20,  17,  10},
//                               {   0,   0,   0,   0,   0},
//                               { -10, -17, -20, -17, -10},
//                               { -10, -17, -20, -17, -10} };
//   static int Xmask [7][7] = { {  -5,  -5,  -5,  0,   5,   5,   5},
//                               { -10, -10, -10,  0,  10,  10,  10},
//                               { -17, -17, -17,  0,  17,  17,  17},
//                               { -20, -20, -20,  0,  20,  20,  20},
//                               { -17, -17, -17,  0,  17,  17,  17},
//                               { -10, -10, -10,  0,  10,  10,  10},
//							   {  -5,  -5,  -5,  0,   5,   5,   5} };
//   static int Ymask [7][7] = { {   5,  10,  17,  20,  17,  10,  5},
//                               {   5,  10,  17,  20,  17,  10,  5},
//                               {   5,  10,  17,  20,  17,  10,  5},
//                               {   0,   0,   0,   0,   0,   0,  0},
//                               {  -5, -10, -17, -20, -17, -10, -5},
//                               {  -5, -10, -17, -20, -17, -10, -5},
//                               {  -5, -10, -17, -20, -17, -10, -5} };

   static int Xmask [9][9] = { {  -3,  -3,  -3,  -3,  0,   3,   3,   3,   3},
                               {  -5,  -5,  -5,  -5,  0,   5,   5,   5,   5},
                               { -10, -10, -10, -10,  0,  10,  10,  10,  10},
                               { -17, -17, -17, -17,  0,  17,  17,  17,  17},
                               { -20, -20, -20, -20,  0,  20,  20,  20,  20},
                               { -17, -17, -17, -17,  0,  17,  17,  17,  17},
                               { -10, -10, -10, -10,  0,  10,  10,  10,  10},
							   {  -5,  -5,  -5, -5,   0,   5,   5,   5,   5},
							   {  -3,  -3,  -3, -3,   0,   3,   3,   3,   3} };
   static int Ymask [9][9] = { {   3,   5,  10,  17,  20,  17,  10,  5,   3},
                               {   3,   5,  10,  17,  20,  17,  10,  5,   3},
                               {   3,   5,  10,  17,  20,  17,  10,  5,   3},
                               {   3,   5,  10,  17,  20,  17,  10,  5,   3},
                               {   0,   0,   0,   0,   0,   0,   0,  0,   0},
                               {  -3,  -5, -10, -17, -20, -17, -10, -5,  -3},
                               {  -3,  -5, -10, -17, -20, -17, -10, -5,  -3},
                               {  -3,  -5, -10, -17, -20, -17, -10, -5,  -3},
                               {  -3,  -5, -10, -17, -20, -17, -10, -5,  -3} };

   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         DeltaX [r][c] = 0;
         DeltaY [r][c] = 0;
   }  }
   MaxMag = 0;
//   for (r=2; r<ImSize-2; r++) {
//      for (c=2; c<ImSize-2; c++) {
//   for (r=3; r<ImSize-3; r++) {
//      for (c=3; c<ImSize-3; c++) {
   for (r=4; r<ImSize-4; r++) {
      for (c=4; c<ImSize-4; c++) {
         dXl = 0;
         dYl = 0;
//         for (y=-2; y<=2; y++) {
//            for (x=-2; x<=2; x++) {
//       for (y=-3; y<=3; y++) {
//          for (x=-3; x<=3; x++) {
         for (y=-4; y<=4; y++) {
            for (x=-4; x<=4; x++) {
               gv = InImage [r+y] [c+x];
 //              dXl += (gv * Xmask [y+2] [x+2]);
 //              dYl += (gv * Ymask [y+2] [x+2]);
 //            dXl += (gv * Xmask [y+3] [x+3]);
 //            dYl += (gv * Ymask [y+3] [x+3]);
               dXl += (gv * Xmask [y+4] [x+4]);
               dYl += (gv * Ymask [y+4] [x+4]);
         }  }
         dX = (int) (dXl/25);
         dY = (int) (dYl/25);
         if (abs(dX) > MaxMag)  MaxMag = abs(dX);
         if (abs(dY) > MaxMag)  MaxMag = abs(dY);
         DeltaX [r][c] = dX;
         DeltaY [r][c] = dY;
   }  }

   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         DeltaX [r][c] = (int) (((long) DeltaX [r][c] * MaxGV) / MaxMag);
         DeltaY [r][c] = (int) (((long) DeltaY [r][c] * MaxGV) / MaxMag);
}  }  }

void HoughTrans (int ImSize, int AccuSize, int MaxGV,
                 BYTE **ThinMag, BYTE **NormMag,
				 int **IntAccu, BYTE **Accu)
{
   int    r,c, rn, cn , Mag, MaxMag, WinSize;
   int Lbound, Rbound, AccuHalf;
   /* quite different algorithm */
   /* Searching for linear shift Xp = X-Xnorm, Yp=X-Ynorm */
   /* Compares only in the window in centre of analysed image 
   with window in reference image arround analysed point 
   sized as accumulator. 
   Analysed window size is ImSize = AccuSize + WinSize */
   WinSize = ImSize - AccuSize;
   Lbound =(ImSize-WinSize)/2;Rbound=(ImSize+WinSize)/2;
   AccuHalf=AccuSize/2;

   for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         IntAccu [r][c] = 0;
         Accu [r][c] = 0;
   }  }
/* accumulation */
   for (r=Lbound ; r<Rbound; r++) {
      for (c=Lbound; c<Rbound; c++) {
         if (ThinMag [r][c]) {
			for (rn=r-AccuHalf ; rn<r+AccuHalf; rn++) {
				for (cn=c-AccuHalf; cn<c+AccuHalf; cn++) {
			 /* goes thrue surrounding of the point in reference image */
			     if (NormMag [rn][cn] > 0x00) {
					 /* Must be stored with offset, point 0,0 
					 is in the centre ie. AccuSize/2, Accusize/2 */
					IntAccu [rn-r+AccuHalf] [cn-c+AccuHalf] ++; }
  }  }  }  }  }
/* Magnitude standardization */
   MaxMag = 0;
   for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         Mag = IntAccu [r][c];
         if (Mag>MaxMag)  MaxMag = Mag;
   }  }

    for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         Mag = IntAccu [r][c];
         if (Mag==MaxMag)  
			 MaxMag = Mag;
   }  }


   if ( MaxMag > 0)
    for (r=0; r<AccuSize; r++) {
      for (c=0; c<AccuSize; c++) {
         Mag = IntAccu [r][c];
         Accu [r][c] = (BYTE) (((long)Mag * MaxGV) / MaxMag);
}  }  }

void Thinning (int ImSize, int DeltaDir,
			   BYTE **GradMag, BYTE **GradAng,
			   BYTE **ThinMag)
/* dependent on number of segments in circle */
/* counts with 256 segments, works with 4 directions */
/* direction use:
0- up
64 - right
128 - down
192 - left 
conditions for write into output:
gradient directions in two adjacent points are lower than DeltaDir
and gradient value in adjacent points is greater than i naalysed one.
Adjacent points are selected with respect of gradient direction */
{
   int  r,c, N1,N2, N1c,N1r, N2c,N2r, N1m,N2m, N1ok,N2ok;
   int  C, Cm, Cmax,Cmin;

   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
         ThinMag [r][c] = 0;
   //      ThinAng [r][c] = 0;
   }  }
   for (r=1; r<ImSize-1; r++) {
      for (c=1; c<ImSize-1; c++)  if (GradMag[r][c]) {
         C = (int) GradAng [r][c];
         if (0<=C && C<=15 || 240<=C && C<=255 || 112<=C && C<=143) {
            N1r = r-1;  N1c = c;
            N2r = r+1;  N2c = c;     /* west, east */

         }else if (16<=C && C<=47 || 144<=C && C<=175) {
            N1r = r-1;  N1c = c-1;
            N2r = r+1;  N2c = c+1;   /* north-east, south-west */

         }else if (48<=C && C<=79 || 176<=C && C<=207) {
            N1r = r;    N1c = c-1;
            N2r = r;    N2c = c+1;   /* north, south */

         }else if (80<=C && C<=111 || 208<=C && C<=239) {
            N1r = r-1;  N1c = c+1;
            N2r = r+1;  N2c = c-1;   /* north-west, south-east */
         }
         Cmin = C - DeltaDir;
         Cmax = C + DeltaDir;
         N1 = GradAng [N1r][N1c];
         N2 = GradAng [N2r][N2c];
         if (Cmin>=0 && Cmax<=255) {
            N1ok = (Cmin<=N1 && N1<=Cmax);
            N2ok = (Cmin<=N2 && N2<=Cmax);
         }else{
            C += 128;  C &= 255;
            Cmin = C - DeltaDir;
            Cmax = C + DeltaDir;
            N1 += 128;  N1 &= 255;  N1ok = (Cmin<=N1 && N1<=Cmax);
            N2 += 128;  N2 &= 255;  N2ok = (Cmin<=N2 && N2<=Cmax);
         }
         if (N1ok && N2ok) {
            N1m = GradMag [N1r][N1c];
            N2m = GradMag [N2r][N2c];
            Cm  = GradMag [r][c];
            if (N1m<=Cm && N2m<=Cm) {
               ThinMag [r][c] = GradMag [r][c];
}  }  }  }  }


int DiscAtan256 (int dy, int dx)
/* circle is divided into 16 parts and gives directions of dy/dx, 
values are in range 0-256 
0 is for interval 0 - 11.25 deg, 16 for 11.25 - 33.75 deg
etc.  to 240. My own modification LiNe*/
{
   int   phi;
   long  quo, Adx, Ady;

   Adx = (long) abs (dx);
   Ady = (long) abs (dy);

   if (Adx==0 || Ady==0)
      return  ((Adx==0 && Ady==0)  ?  0 :
              ((Adx==0) ?
              ((dy < 0) ?  192 : 64) :
              ((dx < 0) ?   128 : 0)));
   else{
      quo = (100*Ady) / Adx;

      phi = ((quo <   20)  ?  0 :
            ((quo <   67)  ? 16 :
            ((quo <  150)  ? 32 :
            ((quo <  503)  ? 48 : 64 ))));

      return ((dy > 0)  ?
             ((dx > 0)  ?  phi   : 128-phi) :     /* 1.quad : 2.quad */
             ((dx < 0)  ?  128+phi :              /* 3.quad */
             ((phi==0)  ?  0     : 256-phi)));   /* 4.quad */
}  }


void Filtr1 (int TLevel, int ImSize,
			  BYTE ** InImage, BYTE ** OutImage, int *Contrast, int *WhCoef)
//			  BYTE ** InImage, BYTE ** OutImage)
{
   /* filter image to black/white with controled relative threshold gray level.
   Used before transformation to simplify transformation */
   int   r,c, MaxMag, MinMag, ThLevel,AvgMag, WhCount;
//   int   r,c, MaxMag, MinMag, ThLevel;
   /* white - FF, black - 00*/
   MaxMag = 0; MinMag = 0xFF; AvgMag= 0; WhCount=0;
//   MaxMag = 0; MinMag = 0xFF;

/* gray limits and avaradge gray levelin image */
   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
			if (InImage [r][c] > MaxMag) MaxMag = InImage [r][c];
			if (InImage [r][c] < MinMag) MinMag = InImage [r][c];
			AvgMag += InImage [r][c];
   }  }

   /* relative thereshold gray level from avaradge grey level*/
   ThLevel = AvgMag/(ImSize*ImSize) ;
   ThLevel += TLevel * (MaxMag - ThLevel)/100;
//   ThLevel = (TLevel*(MaxMag-MinMag)/0xFF)+MinMag;


   /* filtration */
   for (r=0; r<ImSize; r++) {
      for (c=0; c<ImSize; c++) {
		  if (InImage [r][c] > ThLevel) 
		  {OutImage [r][c] = 0xFF; WhCount ++; } else
//			  OutImage [r][c] = 0xFF; else
			  OutImage [r][c] = 0;
   }  }
/* image parameter write (used in output log) */
   *WhCoef = 100 * WhCount/(ImSize*ImSize);
   *Contrast = MaxMag-MinMag;
}
